import { AfterViewInit, ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { UntypedFormControl, UntypedFormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, NavigationEnd, Router } from '@angular/router';
import { forkJoin, of } from 'rxjs';
import { catchError, map } from 'rxjs/operators';
import { PersonalOffer } from '../../models/personalOffer.model';
import { Structure } from '../../models/structure.model';
import { StructureWithOwners } from '../../models/structureWithOwners.model';
import { User } from '../../models/user.model';
import { ProfileService } from '../../profile/services/profile.service';
import { NotificationService } from '../../services/notification.service';
import { PersonalOfferService } from '../../services/personal-offer.service';
import { StructureService } from '../../services/structure.service';
import { MustMatch } from '../../shared/validator/form';
import { CustomRegExp } from '../../utils/CustomRegExp';
import { formUtils } from '../../utils/formUtils';
import { stepType } from '../step.type';
import { accountFormStep } from './account-form/accountFormStep.enum';
import { formType } from './formType.enum';
import { personalOfferFormStep } from './personal-offer-form/personalOfferFormStep.enum';
import { profileFormStep } from './profile-form/profileFormStep.enum';
import { structureFormStep } from './structure-form/structureFormStep.enum';

@Component({
  selector: 'app-form-view',
  templateUrl: './form-view.component.html',
  styleUrls: ['./form-view.component.scss'],
})
export class FormViewComponent implements OnInit, AfterViewInit {
  public routeParam: string;
  public formType = formType;
  public currentPage: stepType;
  public currentFormType: formType;
  public currentForm: UntypedFormGroup;
  public formUtils = new formUtils();
  // Account Form
  public accountForm: UntypedFormGroup;
  public userAcceptNewsletter: boolean;
  // Profile Form
  public profileForm: UntypedFormGroup;
  public isPersonalOfferProfile: boolean;

  // Structure from
  public structureForm: UntypedFormGroup;
  public editForm: UntypedFormGroup;
  public structure: Structure;
  public isEditMode: boolean = false;
  public structureFormStep = structureFormStep;

  // Personal Offers Form
  public personalOfferForm: UntypedFormGroup;
  public hasOtherPersonalOffer: boolean = false;

  // Page and progress var
  public pagesValidation = [];
  public nbSteps: number;

  // Condition form
  public userAcceptSavedDate = false;

  // Collapse var
  public showWebsite: boolean;
  public showSocialNetwork: boolean;

  // Modal canExit var
  public showConfirmationModal = false;
  private resolve: Function;

  public profile: User;
  public isAccountMode: boolean = false;
  public isClaimMode: boolean = false;
  public isJoinMode: boolean = false;
  public claimStructure: boolean = false;
  public linkedStructureId;
  public structureWithOwners: StructureWithOwners;
  public isPageValid: boolean = false;
  public hoursForm: UntypedFormGroup;
  public isRegisterNewMember: boolean = false;

  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private profileService: ProfileService,
    private structureService: StructureService,
    private personalOfferService: PersonalOfferService,
    private notificationService: NotificationService,
    private cdRef: ChangeDetectorRef
  ) {}

  ngAfterViewInit(): void {
    this.cdRef.detectChanges();
  }

  async ngOnInit(): Promise<void> {
    this.routeParam = this.router.routerState.snapshot.url.split('/')[2];

    this.initPage();
    this.profileService.getProfile().then((user: User) => {
      this.profile = user;

      if (!this.isEditMode && this.profile) {
        if (this.profile.structuresLink.length) {
          // if register a new user as a new member structure, no structure to choose
          this.isRegisterNewMember = true;
        }
      }
    });
    // Check if it's a new structure or edit structure
    this.route.data.subscribe((data) => {
      if (data.structure) {
        this.isEditMode = true;
        this.structure = data.structure;
        this.editForm = this.formUtils.createStructureForm(data.structure, this.isEditMode);
        this.structureForm = this.editForm;
        this.hoursForm = this.formUtils.createHoursForm(data.structure);
      }
    });

    this.router.events.subscribe((evt) => {
      if (!(evt instanceof NavigationEnd)) {
        return;
      }
      this.routeParam = this.router.routerState.snapshot.url.split('/')[2];

      this.initPage();
    });
  }

  private initPage(): void {
    const profileFormSteps: number = Object.keys(profileFormStep).length / 2;
    const personnalOfferFormSteps: number = Object.keys(personalOfferFormStep).length / 2 - 1;
    const structureFormSteps: number = Object.keys(structureFormStep).length / 2;
    const totalFormSteps: number = profileFormSteps + personnalOfferFormSteps + structureFormSteps;
    if (formType[this.routeParam] === formType.account) {
      this.nbSteps = 3;
      this.currentPage = accountFormStep.accountInfo;
      this.currentFormType = formType.account;
      this.createAccountForm();
      this.currentForm = this.accountForm;
    }
    if (formType[this.routeParam] === formType.register) {
      this.nbSteps = 3;
      this.currentPage = accountFormStep.accountInfo;
      this.currentFormType = formType.account;
      this.route.data.subscribe((data) => {
        if (data.user) {
          this.createAccountForm(data.user.email);
          this.linkedStructureId = data.user.pendingStructuresLink;
          this.currentForm = this.accountForm;
          this.isAccountMode = true;
        }
      });
    }
    if (formType[this.routeParam] === formType.profile) {
      this.nbSteps = totalFormSteps;
      this.currentPage = profileFormStep.profileBeginningInfo;
      this.currentFormType = formType.profile;
      this.createProfileForm();
      this.currentForm = this.profileForm;
    }
    if (formType[this.routeParam] === formType.structure) {
      if (!this.isEditMode) {
        const PAGE_WITHOUT_COUNT_INCREMENT = 2;
        this.nbSteps = structureFormSteps - PAGE_WITHOUT_COUNT_INCREMENT;
        this.currentPage = structureFormStep.structureChoice;
        this.currentFormType = formType.structure;
        this.structure = new Structure();
        this.structureForm = this.formUtils.createStructureForm(this.structure);
        this.currentForm = this.structureForm;
        // Init hours form
        this.hoursForm = this.formUtils.createHoursForm(this.structure);
      }
    }
    if (formType[this.routeParam] === formType.personaloffer) {
      this.nbSteps = totalFormSteps;
      this.currentPage = personalOfferFormStep.personalOfferAccompaniment;
      this.currentFormType = formType.personaloffer;
      const newPersonalOffer: PersonalOffer = new PersonalOffer();
      this.createPersonalOfferForm(newPersonalOffer);
      this.currentForm = this.personalOfferForm;
    }
  }

  public updateHours(form: UntypedFormGroup): void {
    this.hoursForm = form;
  }

  private createAccountForm(email?: string): void {
    this.accountForm = new UntypedFormGroup(
      {
        email: new UntypedFormControl(email ? email : '', [
          Validators.required,
          Validators.pattern(CustomRegExp.EMAIL),
        ]),
        name: new UntypedFormControl('', [Validators.required, Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER)]),
        surname: new UntypedFormControl('', [
          Validators.required,
          Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER),
        ]),
        phone: new UntypedFormControl('', [Validators.required, Validators.pattern(CustomRegExp.PHONE)]),
        password: new UntypedFormControl('', [
          Validators.required,
          Validators.pattern(CustomRegExp.PASSWORD), //NOSONAR
        ]),
        confirmPassword: new UntypedFormControl(''),
      },
      [MustMatch('password', 'confirmPassword')]
    );
  }

  private createProfileForm(): void {
    this.profileForm = new UntypedFormGroup({
      employer: new UntypedFormGroup({
        name: new UntypedFormControl('', [Validators.required]),
        validated: new UntypedFormControl(false, [Validators.required]),
      }),
      job: new UntypedFormGroup({
        name: new UntypedFormControl('', [Validators.required]),
        validated: new UntypedFormControl(true, [Validators.required]),
        hasPersonalOffer: new UntypedFormControl(true, [Validators.required]),
      }),
      structure: new UntypedFormControl('', [Validators.required]),
    });
  }

  public acceptReceiveNewsletter(isAccepted: boolean): void {
    this.userAcceptNewsletter = isAccepted;
  }

  private createPersonalOfferForm(personalOffer: PersonalOffer): void {
    this.personalOfferForm = new UntypedFormGroup({
      categories: new UntypedFormGroup({
        onlineProcedures: new UntypedFormControl(personalOffer.categories.onlineProcedures),
        baseSkills: new UntypedFormControl(personalOffer.categories.baseSkills),
        advancedSkills: new UntypedFormControl(personalOffer.categories.advancedSkills),
      }),
    });
  }

  public validatePage(value: boolean = true): void {
    this.isPageValid = value;
  }

  public nextPage(): void {
    this.isPageValid = false;
    if (this.currentPage < this.nbSteps) {
      this.currentPage++;
    }
  }
  public prevPage(): void {
    if (this.currentPage > 0) {
      this.currentPage--;
    }
  }
  public displayFooterForm(): boolean {
    if (this.currentPage === profileFormStep.profileBeginningInfo && formType[this.routeParam] === formType.profile)
      return false;
    return true;
  }

  public setHasOtherPersonalOffer(value: boolean): void {
    this.hasOtherPersonalOffer = value;
  }

  public linkStructureToUser(): void {
    this.structure._id = this.structureForm.value._id;
    this.structure.structureName = this.structureForm.value.structureName;

    this.structureService.isClaimed(this.structure._id, this.profile).subscribe((isClaimed) => {
      this.structure.isClaimed = isClaimed;
      if (isClaimed) {
        this.structureService.joinStructure(this.structureForm.value._id, this.profile.email).subscribe(() => {
          this.currentPage = structureFormStep.mailSentInfo;
        });
      } else {
        this.structureService.claimStructureWithAccount(this.structure._id, this.profile.email).subscribe(() => {
          this.currentPage = structureFormStep.mailSentInfo;
        });
      }
    });
  }

  public async endForm(type: { formType: formType; formStep?: stepType }): Promise<void> {
    switch (type.formType) {
      case formType.account:
        break;
      case formType.profile:
        this.saveProfileForm();
        break;
      case formType.structure:
        if (type.formStep === structureFormStep.mailSentInfo) {
          const user = await this.profileService.getProfile();
          if (user.job && user.job.hasPersonalOffer) {
            this.router.navigateByUrl('form/personaloffer');
          } else {
            this.router.navigateByUrl('/');
          }
        }
        if (type.formStep === structureFormStep.structureChoice) {
          // If structure already exist, join it.
          // On first strike the structure._id is not set and is a key to differentiate with other case
          if (this.structureForm.value._id && !this.structure._id) {
            this.linkStructureToUser();
          }
        }
        if (type.formStep === structureFormStep.structureCreationFinishedInfo) {
          this.saveStructureForm();
        }
        if (type.formStep === structureFormStep.noStructure) {
          this.router.navigateByUrl('/profile');
        }
        if (type.formStep === structureFormStep.StructureInfoUnknown) {
          //Creation de coquille vide ??
          this.router.navigateByUrl('/profile');
        }
        break;
      case formType.personaloffer:
        this.savePersonalOfferForm();
        break;
    }
  }

  public saveProfileForm(): void {
    forkJoin({
      employer: this.profileService.createEmployer(this.profileForm.get('employer').value).pipe(
        map((res) => res),
        catchError((_e) => of(this.profileForm.get('employer').value))
      ),
      job: this.profileService.createJob(this.profileForm.get('job').value).pipe(
        map((res) => res),
        catchError((_e) => of(this.profileForm.get('job').value))
      ),
      profile: this.profileService
        .updateProfile(this.profileForm.get('employer').value.name, this.profileForm.get('job').value.name)
        .pipe(
          map((res) => (this.profile = res)),
          catchError((_e) => of())
        ),
    }).subscribe(async () => {
      // if register a new user as a new member structure, no structure to choose
      if (this.isRegisterNewMember) {
        if (this.profile.job && this.profile.job.hasPersonalOffer) {
          this.structureService.getStructure(this.profile.structuresLink[0]).subscribe((structure) => {
            this.structure = new Structure(structure);
            this.router.navigateByUrl('form/personaloffer');
          });
        } else {
          this.router.navigateByUrl('/profile');
        }
      } else {
        this.router.navigateByUrl('form/structure');
      }
    });
  }

  public savePersonalOfferForm(): void {
    this.personalOfferService.createPersonalOffer(this.structure._id, this.personalOfferForm.value).subscribe(() => {
      if (this.hasOtherPersonalOffer) {
        this.router.navigateByUrl('form/structure');
      } else {
        this.nextPage();
      }
    });
  }
  public async saveStructureForm(): Promise<void> {
    const user = await this.profileService.getProfile();
    const newStructure = new Structure(this.structureForm.value);
    newStructure.hours = this.hoursForm.value;
    if (!this.structureForm.value.freeWorkShop) {
      newStructure.freeWorkShop = true;
    }
    this.structureService.createStructure(newStructure, this.profile).subscribe((struct) => {
      if (user.job && user.job.hasPersonalOffer) {
        this.structure = struct;
        this.router.navigateByUrl('form/personaloffer');
      } else {
        this.router.navigateByUrl(`acteurs?id=${struct._id}`);
      }
    });
  }
  public setCurrentStep(step: stepType): void {
    // THIS PROBABLY CREATES CONSOLE ERRORS NG100 only in dev mode, please refer to https://angular.io/errors/NG0100 for more info
    this.isPageValid = false;
    this.currentPage = step;
  }
  public async saveEditedStructure() {
    this.structureService.editStructure(this.getCurrentChanges(this.currentPage), this.structure._id).subscribe(() => {
      this.notificationService.showSuccess('Vos modifications ont bien été prises en compte.', '');
      history.back();
    });
  }

  /**
   * Returns only fields on the current step for edition.
   * We have to do this because some fields might not be completed and we cannot send the full object because of BO validation
   */
  public getCurrentChanges(currentPage: stepType): Partial<Structure> {
    switch (currentPage) {
      case structureFormStep.structureChoice:
        return {
          _id: this.structureForm.get('_id').value,
        };
      case structureFormStep.structureNameAndAddress:
        return {
          structureName: this.structureForm.get('structureName').value,
          address: this.structureForm.get('address').value,
        };
      case structureFormStep.structureContact:
        return {
          contactPhone: this.structureForm.get('contactPhone').value,
          contactMail: this.structureForm.get('contactMail').value,
        };
      case structureFormStep.structureAccessModality:
        return {
          categories: {
            accessModality: this.structureForm.get('categories').get('accessModality').value,
          },
        };
      case structureFormStep.structureHours:
        return {
          hours: this.hoursForm.value,
          exceptionalClosures: this.structureForm.get('exceptionalClosures').value,
        };
      case structureFormStep.structurePmr:
        return {
          pmrAccess: this.structureForm.get('pmrAccess').value,
        };
      case structureFormStep.structureWebAndSocialNetwork:
        return {
          facebook: this.structureForm.get('facebook').value,
          instagram: this.structureForm.get('instagram').value,
          linkedin: this.structureForm.get('linkedin').value,
          twitter: this.structureForm.get('twitter').value,
          website: this.structureForm.get('website').value,
        };
      case structureFormStep.structurePublicTarget:
        return {
          categories: {
            age: this.structureForm.get('categories').get('age').value,
            languageAndIlliteracy: this.structureForm.get('categories').get('languageAndIlliteracy').value,
            handicaps: this.structureForm.get('categories').get('handicaps').value,
            publicOthers: this.structureForm.get('categories').get('publicOthers').value,
          },
        };
      case structureFormStep.structureDigitalHelpingAccompanimentOther:
        return {
          otherDescription: this.structureForm.get('otherDescription').value,
        };
      case structureFormStep.structureDigitalHelpingAccompaniment:
        return {
          categories: {
            onlineProcedures: this.structureForm.get('categories').get('onlineProcedures').value,
          },
        };
      case structureFormStep.structureTrainingType:
        return {
          categories: {
            baseSkills: this.structureForm.get('categories').get('baseSkills').value,
            advancedSkills: this.structureForm.get('categories').get('advancedSkills').value,
          },
        };
      case structureFormStep.structureTrainingPrice:
        return {
          freeWorkShop: this.structureForm.get('freeWorkShop').value,
        };
      case structureFormStep.structureOtherServices:
        return {
          categories: {
            equipmentsServices: this.structureForm.get('categories').get('equipmentsAndServices').value,
          },
        };
      case structureFormStep.structureWifi:
        return {
          categories: {
            selfServiceMaterial: this.structureForm.get('categories').get('selfServiceMaterial').value,
          },
        };
      case structureFormStep.structureEquipments:
        return {
          nbComputers: this.structureForm.get('nbComputers').value,
          nbPrinters: this.structureForm.get('nbPrinters').value,
          nbScanners: this.structureForm.get('nbScanners').value,
        };
      case structureFormStep.structureLabels:
        return {
          categories: {
            labelsQualifications: this.structureForm.get('categories').get('labelsQualifications').value,
          },
        };
      case structureFormStep.structureDescription:
        return {
          description: this.structureForm.get('description').value,
        };
      case structureFormStep.structureCovidInfo:
        return {
          lockdownActivity: this.structureForm.get('lockdownActivity').value,
        };
      case structureFormStep.structureConsent:
        return {
          dataShareConsentDate: this.structureForm.get('dataShareConsentDate').value ? new Date().toString() : null,
        };
    }
  }

  public canExit(): Promise<boolean> {
    // List all exit pages in order to authorise exit
    const exitPages: stepType[] = [
      structureFormStep.noStructure,
      structureFormStep.structureCreationFinishedInfo,
      structureFormStep.mailSentInfo,
      profileFormStep.profileJobSelection,
      personalOfferFormStep.personalOfferFinishedInfo,
    ];
    // Avoid confirmation when user submit form and leave.
    if (
      this.currentPage === this.nbSteps ||
      this.currentPage < 1 ||
      this.isEditMode ||
      exitPages.includes(this.currentPage)
    ) {
      return Promise.resolve(true);
    } else {
      return new Promise((resolve) => this.showModal(resolve));
    }
  }
  private showModal(resolve: Function): void {
    this.showConfirmationModal = true;
    this.resolve = resolve;
  }
  public hasRedirectionAccepted(hasAccept: boolean): void {
    this.resolve(hasAccept);
    this.showConfirmationModal = false;
  }
}
