import { StructureCategoryEnum } from '../shared/enum/structureCategory.enum';
import { StructureCategoryIconEnum } from '../shared/enum/structureCategoryIcon.enum';
import { StructureTypeEnum } from '../shared/enum/structureType.enum';
import { Equipment } from '../structure-list/enum/equipment.enum';
import { Weekday } from '../structure-list/enum/weekday.enum';
import { Address } from './address.model';
import { Day } from './day.model';
import { OpeningDay } from './openingDay.model';
import { PersonalOffer } from './personalOffer.model';
import { StructureType } from './structureType.model';
import { Week } from './week.model';

export class Structure {
  public _id: string = null;
  public numero: string = null;
  public createdAt: string = null;
  public updatedAt: string = null;
  public toBeDeletedAt: string = null;
  public structureName: string = null;
  public structureType: StructureType = null;
  public description: string = null;
  public address: Address = new Address();
  public contactPhone: string = null;
  public contactMail: string = null;
  public website: string = null;
  public facebook: string = null;
  public twitter: string = null;
  public instagram: string = null;
  public linkedin: string = null;
  public lockdownActivity: string = null;
  public pmrAccess: boolean = null;
  public placeOfReception: boolean = null;
  public choiceCompletion: boolean = null;
  public contactPersonFirstName: string = null;
  public contactPersonLastName: string = null;
  public contactPersonEmail: string = null;
  public categories: { [key: string]: string[] };
  public categoriesDisplay: { [key: string]: string[] };
  public remoteAccompaniment: boolean = null;
  public nbComputers: number = null;
  public nbPrinters: number = null;
  public nbScanners: number = null;
  public exceptionalClosures: string = null;
  public hours: Week;
  public freeWorkShop: boolean = null;
  public otherDescription: string = null;

  public isOpen: boolean = false;
  public openedOn: OpeningDay = new OpeningDay();

  public distance?: number;
  public coord?: number[] = [];
  public dataShareConsentDate?: string;

  public accountVerified: boolean = false;

  public personalOffers: PersonalOffer[] = [];

  public alreadySelected? = false;
  public isClaimed?: boolean = null;
  public idCNFS?: string;

  constructor(obj?: any) {
    Object.assign(this, obj, {
      hours: obj && obj.hours ? new Week(obj.hours) : new Week(),
      categories: {
        accessModality: obj?.categories?.accessModality || [],
        advancedSkills: obj?.categories?.advancedSkills || [],
        age: obj?.categories?.age || [],
        baseSkills: obj?.categories?.baseSkills || [],
        handicaps: obj?.categories?.handicaps || [],
        labelsQualifications: obj?.categories?.labelsQualifications || [],
        languageAndIlliteracy: obj?.categories?.languageAndIlliteracy || [],
        onlineProcedures: obj?.categories?.onlineProcedures || [],
        publicOthers: obj?.categories?.publicOthers || [],
        selfServiceMaterial: obj?.categories?.selfServiceMaterial || [],
        equipmentsServices: obj?.categories?.equipmentsServices || [],
      },
    });
  }

  public getDayhours(day: Weekday): Day {
    switch (day) {
      case Weekday.monday:
        return this.hours.monday;
      case Weekday.tuesday:
        return this.hours.tuesday;
      case Weekday.thursday:
        return this.hours.thursday;
      case Weekday.wednesday:
        return this.hours.wednesday;
      case Weekday.friday:
        return this.hours.friday;
      case Weekday.saturday:
        return this.hours.saturday;
      case Weekday.sunday:
        return this.hours.sunday;
      default:
        return null;
    }
  }

  /**
   * Check if a structure has equipments
   */
  public hasEquipments(): boolean {
    if (this.categories.selfServiceMaterial.length && this.hasNotOnlyEmptyEquipments()) {
      return true;
    }
    return false;
  }

  /**
   * Verify that a structure as not only equipments with 0 as value. This is mostly use for display.
   * @returns {Boolean} validation
   */
  public hasNotOnlyEmptyEquipments(): boolean {
    if (this.nbComputers + this.nbPrinters + this.nbScanners > 0) return true;
    return false;
  }

  /**
   * Check if a structure has pass Numeric label
   */
  public hasPassNumeric(): boolean {
    return this.categories.labelsQualifications.includes('passNumerique');
  }

  /**
   * Return a range, according to the distance, between [1,3] to get a distance reference.
   * - [0,5km] => 1
   * - [5km,10km] => 2
   * - [10km, [ => 3
   */
  public getDistanceRange(): number {
    if (!this.distance) {
      return 3;
    } else {
      // If it's in km
      if (this.distance > 10000) {
        return 3;
      } else if (this.distance < 5000) {
        // If it's between 0 and 500 m
        return 1;
      } else {
        return 2;
      }
    }
  }

  public getLat(): number {
    return this.coord[1];
  }

  public getLon(): number {
    return this.coord[0];
  }

  public getEquipmentsIcon(equipment: Equipment): string {
    switch (equipment) {
      case Equipment.wifi:
        return 'wifiEnAccesLibre';
      case Equipment.printer:
        return 'print';
      case Equipment.computer:
        return 'computer';
      case Equipment.scanner:
        return 'scan';
      default:
        return null;
    }
  }

  public getEquipmentsTitle(equipment: Equipment): string {
    switch (equipment) {
      case Equipment.wifi:
        return 'Wifi en accès libre';
      case Equipment.printer:
        return this.nbPrinters > 1 ? 'Imprimantes' : 'Imprimante';
      case Equipment.computer:
        return this.nbComputers > 1 ? 'Ordinateurs' : 'Ordinateur';
      case Equipment.scanner:
        return this.nbScanners > 1 ? 'Scanners' : 'Scanner';
      default:
        return null;
    }
  }

  public getLabelTypeStructure(): StructureTypeEnum | '' {
    if (this.structureType) return StructureTypeEnum[this.structureType.value] || '';
    return '';
  }

  public getTypeStructureIcon(): StructureCategoryIconEnum {
    switch (this.structureType?.category) {
      case StructureCategoryEnum.public:
        return StructureCategoryIconEnum['public'];
      case StructureCategoryEnum.private:
        return StructureCategoryIconEnum['private'];
      case StructureCategoryEnum.privateLucrative:
        return StructureCategoryIconEnum['privateLucrative'];
      default:
        return StructureCategoryIconEnum['public'];
    }
  }

  public hasSocialNetwork(): boolean {
    return (
      (this.facebook !== null && this.facebook !== '') ||
      (this.instagram !== null && this.instagram !== '') ||
      (this.linkedin !== null && this.linkedin !== '') ||
      (this.twitter !== null && this.twitter !== '')
    );
  }
}
